/*
** Module   : ServerExports.java
** Abstract : defines all UI-related API's exported by the server for use
**            by the client  
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 2005xxxx   @23594 Created initial version.
** 002 NVS 20060130   @24149 Server exports taking screen buffer, now 
**                           return screen buffer as well. These are: 
**                           trigger(), validate().
** 003 SIY 20060130   @24156 Minor cleanups.
** 004 NVS 20060221   @24622 Added help() export that locates and runs a
**                           preconfigured help application.
** 005 GES 20060226   @24744 Added a query allowing the client to request
**                           the latest screen buffer from the server.
**                           This is useful in cases where the client
**                           initiates the view() of a frame without an
**                           explicit call from the server (e.g. page
**                           headers and footers in redirected output
**                           mode).
** 006 SIY 20060315   @25139 Changed signatures of BROWSE-related methods.
** 007 NVS 20060320   @25158 Simplified validation protocol by using new
**                           class ValidationResult.
** 008 GES 20060808   @28492 Use int instead of long for the browse 
**                           methods.
** 009 GES 20061017   @30452 Added a new option to trigger().
** 010 GES 20061030   @30788 Changed the return value from trigger() to
**                           eliminate the extra call to the client
**                           (to mark an event as "consume") and to make
**                           the trigger processing synchronous. This
**                           also makes it safer for nested triggers.
** 011 SIY 20061105   @30985 Added getCurrentRow() method.
** 012 NVS 20070118   @31889 EventList class is used instead of WaitList.
** 013 NVS 20070328   @32631 help() request from the client should report
**                           the lastKey variable.
** 014 SIY 20070504   @33426 Updated getRowCount().
** 015 SIY 20111101          Updated imports.
** 016 CA  20130822          Added support of appserver async requests: posting and executing of
**                           server-side events on the client side.
** 017 CA  20130908          Added support for socket runtime.
** 018 HC  20131031          Changed resource ID data type to Long, see issue #2183.
** 019 CA  20131218          Fixed H221 - fixed the signature for createSocket.
** 020 CA  20140711          Enhanced font support, for GUI clients.
** 021 CA  20140728          Added notification that a frame was hidden by the client-side.
** 022 CA  20140805          Added API to load a image from server-side.
** 023 MAG 20140812          Implements COLOR-TABLE, for GUI clients.
** 024 MAG 20140911          Added load environment API.
** 025 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 026 EVL 20141204          Differentiate window icon loading from other images.
** 027 EVL 20141216          Adding SESSION:TOOLTIPS attribute handling.
** 028 EVL 20150114          Adding the API to get application icon data from the server side.
**                           Rework icon/image implementation.
** 029 CA  20150122          Changed trigger API to be aware of the non-widget events (the 
**                           resource ID).
** 030 CA  20150212          Allow events to be captured and processed by the widgets without
**                           server-side or main event-loop notification.  This fixes problems in
**                           the EDITING and WAIT-FOR event loops, for i.e. COMBO-BOX and EDITOR
**                           widgets.
** 031 CA  20150220          Fixed the FRAME-VALUE statement: if HELP is active, it references the
**                           FOCUSed frame/widget as before the HELP is invoked.
** 032 SVL 20150212          Modified getRows signature.
** 033 CA  20150317          Added API to determine if a legacy font is fixed-width.
** 034 SVL 20150411          Added isRowAvailable.
*/

package com.goldencode.p2j.ui;

import java.util.*;

import com.goldencode.p2j.ui.ColorTable.EnvironmentColorTable;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.util.*;

/**
 * Defines all UI-related API's exported by the server for use by the
 * client.
 */
public interface ServerExports
{
   /**
    * Called from the client-side, it executes the server-side event by calling 
    * {@link ServerEvent#invoke(int)}.
    * 
    * @param    id
    *           The id of the server event.
    */
   public void invokeServerEvent(int id);
   
   /**
    * Check if the resource with the given id is still valid to be used in a WAIT-FOR statement.
    * It will return <code>false</code> if:
    * <ul>
    *    <li>there is no resource associated with the given id or the resource is unknown.</li>
    *    <li>the SENSITIVE attribute for the resource is <code>false</false>.</li>
    *    <li>the resource is not valid</li>.
    * </ul>
    * In all other cases, it will return <code>true</code>
    * 
    * @param    id
    *           The resource id.
    * 
    * @return   See above.
    */
   public boolean validResource(long id);
   
   /**
    * Signals a trigger to the application.  If a trigger is registered for
    * this event and source widget combination, then it will be executed.
    * 
    * @param    triggerId
    *           The trigger to be called.
    * @param    eventId
    *           The event which was generated.
    * @param    sourceId
    *           The widget which generated the event.
    * @param    resourceId
    *           The resource which generated the event.
    * @param    sb
    *           The associated frame's current <code>ScreenBuffer</code>
    *
    * @return   A container including the <code>ScreenBuffer</code> 
    *           representing the latest screen values of all widgets and the
    *           consume flag.
    */
   public TriggerReturnValue trigger(int          triggerId,
                                     int          eventId,
                                     int          sourceId,
                                     long         resourceId,
                                     ScreenBuffer sb);
   
   /**
    * Signals a validation event to the application.
    * <p>
    * This signal is used to trigger validation execution on the server.
    *
    * @param   sourceId
    *          widget ID of the widget generated the trigger.
    * @param   sb
    *          current <code>ScreenBuffer</code>
    * 
    * @return  <code>ValidationResult</code> representing the validation  
    *          result, error message and screen value changes.
    */
   public ValidationResult validate(int sourceId, ScreenBuffer sb);
   
   /**
    * Locates and executes a preconfigured HELP application.
    *
    * @param   lastKey
    *          The most recent key event code
    * @param   buffer
    *          The screen buffer associated with the FOCUS as before HELP is invoked.
    */
   public void help(int lastKey, ScreenBuffer buffer);

   /**
    * Obtains the latest screen buffer associated with the given frame.
    *
    * @param   frameId
    *          The frame for which the screen buffer should be returned.
    *
    * @return  <code>ScreenBuffer</code> representing the current screen 
    *          values of all widgets in the given frame.
    */
   public ScreenBuffer refresh(int frameId);   
   
   //------------------------------------------------------------------------
   // BrowseModel calls
   //------------------------------------------------------------------------
   /**
    * Return information about columns.
    * 
    * @param   browseId
    *          ID of the browse which should handle the request.
    * 
    * @return  A meta information about columns.
    */
   public BrowseColumnConfig[] getColumns(int browseId);

   /**
    * Get number of available rows starting from the given one.
    *
    * @param   browseId
    *          ID of the browse which should handle the request.
    * @param   startRow
    *          Initial row to start count rows from.
    *
    * @return  Number of available rows.
    */
   public int getRowCount(int browseId, int startRow);

   /**
    * Retrieves several subsequent rows from the database.
    *
    * @param    browseId
    *           ID of the browse which should handle the request.
    * @param    startRowIndex
    *           Index of the first row to retrieve.
    * @param    count
    *           Maximum count of rows to return.
    * @param    forward
    *           <code>true</code> if rows are retrieved in the forward direction,
    *           <code>false</code> if rows are retrieved in the backward direction,
    *
    * @return   Array of rows. Each row is array of cell values, but the first element of the row
    *           array is 0-based index of the row in the browse model.
    *           Note that if rows are fetched in the backward direction, the resulting rows are
    *           returned in the straight order starting from the row with the least row number.
    *           If no query specified, an array of size 0 is returned.
    */
   public BaseDataType[][] getRows(int browseId,
                                   int startRowIndex,
                                   int count,
                                   boolean forward);
   
   /**
    * Updates selected cells in the given row.
    * 
    * @param   browseId
    *          ID of the browse which should handle the request.
    * @param   rowIndex
    *          Index of the row to update.
    * @param   cellIndices
    *          Cells for update indexes.
    * @param   cellValues
    *          Cells for update new values.
    * 
    * @throws  DataValidationException
    *          New values don't passed database validation rules.
    */
   public void updateRow(int            browseId,
                         int            rowIndex,
                         int[]          cellIndices,
                         BaseDataType[] cellValues)
   throws DataValidationException;

   /**
    * Returns <code>true</code> if specified row is new and is not present
    * in the data base.
    * 
    * @param   browseId
    *          ID of the browse which should handle the request.
    * @param   rowIndex
    *          Index of the row to return information about.
    * @return  <code>true</code> if row is new and <code>false</code>
    *          otherwise.
    */
   public boolean isNewRow(int browseId, int rowIndex);

   /**
    * Inform server about current row in the browse.
    * 
    * @param   browseId
    *          ID of the browse which should handle the request.
    * @param   rowIndex
    *          0-based index of the current row.
    *          
    * @return  New current row.
    */
   public int setCurrentRow(int browseId, int rowIndex);

   /**
    * Get current row in the query.
    *
    * @param   browseId
    *          ID of the browse which should handle the request.
    *          
    * @return  The current row index.
    */
   public int getCurrentRow(int browseId);

   /**
    * Determines if the specified browse row is "available", i.e. it is not deleted.
    *
    * @param  browseId
    *         ID of the browse which should handle the request.
    * @param  rowIndex
    *         0-base index of the target row in the browse model.
    *
    * @return <code>true</code> if the row is "available", i.e. it is not deleted.
    */
   public boolean isRowAvailable(int browseId, int rowIndex);
   
   /**
    * Called by the P2J client side, it will inform the server that a server-event was generated.
    * Although the event originates from the P2J Client Side, socket events are server-side 
    * events.
    * 
    * @param    resourceId
    *           The resource ID for the socket to which the READ-RESPONSE event belongs.
    *           
    * @return   The ID of the generated server event or <code>-1</code> if a event could not be
    *           posted (i.e. the resource is not SENSITIVE).
    */
   public int readResponseEvent(long resourceId);
   
   /**
    * Called by the P2J client side, it will inform the server that a server event was generated.
    * Although the event originates from the P2J Client Side, socket events are server-side 
    * events.
    * 
    * @param    resourceId
    *           The resource ID for the server socket to which the CONNECT event belongs.
    * @param    clientSocketId
    *           The resource ID of the socket created for the new connected accepted by the
    *           server socket.
    *           
    * @return   The ID of the generated server event or <code>-1</code> if a event could not be
    *           posted (i.e. the resource is not SENSITIVE).
    */
   public int connectEvent(long resourceId, long clientSocketId);
   
   /**
    * Create a socket and associated with the given ID.  The received ID is not a resource ID; 
    * this represents the ID of the associated {@link LowLevelSocketImpl} instance.
    *   
    * @param    id
    *           The ID of the associated {@link LowLevelSocketImpl} instance.
    *           
    * @return   The ID of the created {@link SocketImpl} resource.
    */
   public long createSocket(int id);

   /**
    * Read the font-table with the name of the legacy system fonts.
    * 
    * @return   The font table.  As 4GL does, this is 0-indexed.
    */
   public String[] readSystemFontTable();
   
   /**
    * Read the entire font-table.
    * 
    * @return   The font table.  As 4GL does, this is 0-indexed.
    */
   public String[] readFontTable();

   /**
    * Read the default fonts.
    * 
    * @return   An array with the font details, with default font on index o and default fixed
    *           font on index 1.
    */
   public String[] readDefaultFonts();

   /**
    * Get the legacy metrics for the given font, using the font specified by the given key.
    * 
    * @param    key
    *           The font's key.
    *           
    * @return   The legacy metrics, with width on index 0 and height on index 1.
    */
   public int[] getLegacyFontMetrics(String key);

   /**
    * Check if the given font face name is for a fixed font, registered in the {@link #fixedFonts}
    * set.
    * 
    * @param   name
    *          The font name.
    */
   public boolean isLegacyFixedFont(String name);
   
   /**
    * Get the legacy metrics for the given text, using the font specified by the given key.
    * 
    * @param    text
    *           The text.
    * @param    key
    *           The font's key.
    *           
    * @return   The legacy metrics, with width on index 0 and height on index 1.
    */
   public int[] getLegacyTextMetrics(String text, String key);

   /**
    * Get the custom definition of this font.  If none available, return <code>null</code>.
    * 
    * @param    name
    *           The font name.
    *           
    * @return   A byte array with the font's definition, else <code>null</code>.
    */
   public byte[] getCustomFontDefinition(String name);
   
   /**
    * Get color table of the specified environment.
    * 
    * @param   envName
    *          Environment name. Could be <code>null</code> if is default environment. 
    *          
    * @return  see above.
    */
   public EnvironmentColorTable readColorTable(String envName);
   
   /**
    * Get system colors.
    * 
    * @return  System colors
    */
   public ColorTableEntry[] readSystemColorTable();

   /**
    * Load the specified environment as resource.  The environment is stored in an INI file
    * embedded as a resource inside application jar file.
    * 
    * @param    env
    *           The INI file source.
    * 
    * @return   The environment file bytes, or {@code null} if the environment file could not be
    *           loaded.
    */
   public byte[] loadEnvironment(String env);

   /**
    * Synchronize the field IDs for the server-side widget configurations with the given one.
    * 
    * @param    configs
    *           A map of configurations, having as key a class name and as value a list of field
    *           names (the order in this list gives the field ID).
    */
   public void initWidgetConfigurations(Map<String, List<String>> configs);
   
   /**
    * Accesses the SESSION:TOOLTIPS attribute for this session.
    *
    * @return   <code>true</code> if tooltips are available for this session
    *           <code>false</code> otherwise.
    */
   public boolean getSessionTooltips();
   
   /**
    * Refresh the server-side screen buffer for the given frame.
    * 
    * @param    buff
    *           The screen buffer data to be refreshed.  May be <code>null</code>.
    */
   public void refreshBuffer(ScreenBuffer buff);
}
